@@ -1,5 +1,4 @@
-"""A convenience which constructs expression trees from an easy-to-read
-EBNF-like syntax
+"""A convenience which constructs expression trees from an easy-to-read syntax
 
 Use this unless you have a compelling reason not to; it performs some
 optimizations that would be tedious to do when constructing an expression tree
@@ -8,15 +7,17 @@ by hand.
 """
 import ast
 
-from parsimonious.exceptions import UndefinedLabel
-from parsimonious.expressions import *
+from parsimonious.exceptions import BadGrammar, UndefinedLabel
+from parsimonious.expressions import (Literal, Regex, Sequence, OneOf, AllOf,
+    Optional, ZeroOrMore, OneOrMore)
 from parsimonious.nodes import NodeVisitor
+from parsimonious.utils import StrAndRepr
 
 
 __all__ = ['Grammar']
 
 
-class Grammar(dict):
+class Grammar(StrAndRepr, dict):
     """A collection of expressions that describe a language
 
     You can start parsing from the default expression by calling ``parse()``
@@ -45,27 +46,26 @@ class Grammar(dict):
       increase cache hit ratio. [Is this implemented yet?]
 
     """
-    def __init__(self, dsl, default_rule=None):
+    def __init__(self, rules, default_rule=None):
         """Construct a grammar.
 
+        :arg rules: A string of production rules, one per line. There must be
+            at least one rule.
         :arg default_rule: The name of the rule invoked when you call
             ``parse()`` on the grammar. Defaults to the first rule.
 
         """
-        # Maybe we should keep the original DSL text around in case people want
-        # to extend already-compiled grammars. We can't rely on callers to
-        # nicely expose their DSL strings. We can either have extending callers
-        # pull the text off Grammar.dsl, or we could get fancy and define
-        # __add__ on Grammars and strings. Or maybe, if you want to extend a
-        # grammar, just prepend (or append?) your string to its, and yours will
-        # take precedence. Or use the OMeta delegation syntax. Or yield back
-        # the dynamic reconstruction of the DSL.
-        rules, first = self._rules_from_dsl(dsl)
+        # We can either have extending callers pull the rule text out of repr,
+        # or we could get fancy and define __add__ on Grammars and strings. Or
+        # maybe, if you want to extend a grammar, just prepend (or append?)
+        # your string to its, and yours will take precedence. Or use the OMeta
+        # delegation syntax.
+        exprs, first = self._expressions_from_rules(rules)
 
-        self.update(rules)
-        self.default_rule = rules[default_rule] if default_rule else first
+        self.update(exprs)
+        self.default_rule = exprs[default_rule] if default_rule else first
 
-    def _rules_from_dsl(self, dsl):
+    def _expressions_from_rules(self, rules):
         """Return a 2-tuple: a dict of rule names pointing to their
         expressions, and then the first rule.
 
@@ -75,27 +75,54 @@ class Grammar(dict):
         multiple roots.
 
         """
-        tree = dsl_grammar.parse(dsl)
-        return DslVisitor().visit(tree)
+        tree = rule_grammar.parse(rules)
+        if tree is None:
+            raise BadGrammar('There is an error in your rule definitions. '
+                             'Sorry for the vague error reporting at the '
+                             'moment.')
+
+        return RuleVisitor().visit(tree)
 
     def parse(self, text):
         """Parse some text with the default rule."""
         return self.default_rule.parse(text)
 
+    def match(self, text, pos=0):
+        """Parse some text with the default rule but not necessarily all the
+        way to the end.
+
+        :arg pos: The index at which to start parsing
+
+        """
+        return self.default_rule.match(text, pos=pos)
+
+    def __unicode__(self):
+        """Return a rule string that, when passed to the constructor, would
+        reconstitute the grammar."""
+        exprs = [self.default_rule]
+        exprs.extend(expr for expr in self.itervalues() if
+                     expr is not self.default_rule)
+        return '\n'.join(expr.as_rule() for expr in exprs)
+
+    def __repr__(self):
+        """Return an expression that will reconstitute the grammar."""
+        return "Grammar('%s')" % str(self).encode('string_escape')
+
 
 class BootstrappingGrammar(Grammar):
-    """The grammar used to recognize the DSL that describes other grammars
+    """The grammar used to recognize the textual rules that describe other
+    grammars
 
     This grammar gets its start from some hard-coded Expressions and claws its
     way from there to an expression tree that describes how to parse the
-    grammar description DSL.
+    grammar description syntax.
 
     """
-    def _rules_from_dsl(self, dsl):
-        """Return the rules for parsing the grammar DSL.
+    def _expressions_from_rules(self, rule_syntax):
+        """Return the rules for parsing the grammar definition syntax.
 
-        Return a 2-tuple: a dict of rule names pointing to their
-        expressions, and then the first rule.
+        Return a 2-tuple: a dict of rule names pointing to their expressions,
+        and then the top-level expression for the first rule.
 
         """
         # Hard-code enough of the rules to parse the grammar that describes the
@@ -113,11 +140,9 @@ class BootstrappingGrammar(Grammar):
         another_term = Sequence(_, term, name='another_term')
         sequence = Sequence(term, OneOrMore(another_term), name='sequence')
         or_term = Sequence(_, Literal('/'), another_term, name='or_term')
-        or_terms = OneOrMore(or_term, name='or_terms')
-        ored = Sequence(term, or_terms, name='ored')
+        ored = Sequence(term, OneOrMore(or_term), name='ored')
         and_term = Sequence(_, Literal('&'), another_term, name='and_term')
-        and_terms = OneOrMore(and_term, name='and_terms')
-        anded = Sequence(term, and_terms, name='anded')
+        anded = Sequence(term, OneOrMore(and_term), name='anded')
         poly_term = OneOf(anded, ored, sequence, name='poly_term')
         rhs = OneOf(poly_term, term, name='rhs')
         eol = Regex(r'[\r\n$]', name='eol')  # TODO: Support $.
@@ -125,14 +150,14 @@ class BootstrappingGrammar(Grammar):
                         Optional(_), rhs, Optional(_), eol, name='rule')
         rules = Sequence(OneOrMore(rule), Optional(ws), name='rules')
 
-        # Use those hard-coded rules to parse the (possibly more extensive) DSL
-        # grammar definition. (For example, unless I start using parentheses in
-        # the DSL definition itself, I should never have to hard-code
+        # Use those hard-coded rules to parse the (possibly more extensive)
+        # rule syntax. (For example, unless I start using parentheses in the
+        # rule language definition itself, I should never have to hard-code
         # expressions for those above.)
-        dsl_tree = rules.parse(dsl)
+        rule_tree = rules.parse(rule_syntax)
 
         # Turn the parse tree into a map of expressions:
-        return DslVisitor().visit(dsl_tree)
+        return RuleVisitor().visit(rule_tree)
 
 
 # The grammar for parsing PEG grammar definitions:
@@ -141,19 +166,17 @@ class BootstrappingGrammar(Grammar):
 # This is a nice, simple grammar. We may someday add parentheses or support for
 # multi-line rules, but it's a safe bet that the future will always be a
 # superset of this.
-dsl_text = (r'''
+rule_syntax = (r'''
     rules = rule+ ws?
     rule = ws? label _? "=" _? rhs _? eol
     literal = ~"u?r?\"[^\"\\\\]*(?:\\\\.[^\"\\\\]*)*\""is
     eol = ~r"(?:[\r\n]|$)"
     rhs = poly_term / term
     poly_term = anded / ored / sequence
-    anded = term and_terms
+    anded = term and_term+
     and_term = _ "&" another_term
-    and_terms = and_term+
-    ored = term or_terms
     or_term = _ "/" another_term
-    or_terms = or_term+
+    ored = term or_term+
     sequence = term another_term+
     another_term = _ term
     not_term = "!" term'''  # TODO: Half thought out. Make this work.
@@ -172,10 +195,11 @@ dsl_text = (r'''
 
 
 class LazyReference(unicode):
-    """A lazy reference to a rule, which we resolve after grokking all the rules"""
+    """A lazy reference to a rule, which we resolve after grokking all the
+    rules"""
 
 
-class DslVisitor(NodeVisitor):
+class RuleVisitor(NodeVisitor):
     """Turns a parse tree of a grammar definition into a map of ``Expression``
     objects
 
@@ -185,6 +209,8 @@ class DslVisitor(NodeVisitor):
     """
     quantifier_classes = {'?': Optional, '*': ZeroOrMore, '+': OneOrMore}
 
+    visit_rhs = visit_poly_term = visit_term = visit_atom = NodeVisitor.lift_child
+
     def visit_quantified(self, quantified, (atom, quantifier)):
         return self.quantifier_classes[quantifier.text](atom)
 
@@ -194,14 +220,10 @@ class DslVisitor(NodeVisitor):
         rhs.name = label  # Assign a name to the expr.
         return rhs
 
-    def visit_rhs(self, rhs, (term_or_poly,)):
-        """Lift the ``term`` or ``poly_term`` up to replace this node."""
-        return term_or_poly
-
-    def visit_sequence(self, sequence, (term, one_or_more_other_terms)):  # TODO: right? I'm tired right now. How do the children get passed in? [I think this is right.]
+    def visit_sequence(self, sequence, (term, other_terms)):
         """A parsed Sequence looks like [term node, OneOrMore node of
         ``another_term``s]. Flatten it out."""
-        return Sequence(term, *one_or_more_other_terms)
+        return Sequence(term, *other_terms)
 
     def visit_ored(self, ored, (first_term, other_terms)):
         return OneOf(first_term, *other_terms)
@@ -214,10 +236,6 @@ class DslVisitor(NodeVisitor):
         """
         return term
 
-    def visit_or_terms(self, or_terms, or_term_children):
-        """Raise the list of children up to replace the OneOrMore node around them."""
-        return or_term_children
-
     def visit_anded(self, anded, (first_term, other_terms)):
         return AllOf(first_term, *other_terms)
 
@@ -229,12 +247,9 @@ class DslVisitor(NodeVisitor):
         """
         return term
 
-    def visit_and_terms(self, and_terms, and_term_children):
-        """Raise the list of children up to replace the OneOrMore node around them."""
-        return and_term_children
-
     def visit_another_term(self, another_term, (_, term)):
-        """Strip off the space, and just return the actual term involved in ``another_term``.
+        """Strip off the space, and just return the actual term involved in
+        ``another_term``.
 
         This lets us avoid repeating the stripping of the leading space in
         ``visit_or_term``, ``visit_and_term``, and elsewhere.
@@ -242,10 +257,6 @@ class DslVisitor(NodeVisitor):
         """
         return term
 
-    def visit_poly_term(self, poly_term, (anded_ored_or_sequence,)):
-        """Lift up the only child of a ``poly_term`` to take its place."""
-        return anded_ored_or_sequence
-
     def visit_label(self, label, visited_children):
         """Stick a :class:`LazyReference` in the tree as a placeholder.
 
@@ -254,14 +265,6 @@ class DslVisitor(NodeVisitor):
         """
         return LazyReference(label.text)
 
-    def visit_term(self, term, (quantified_or_atom,)):
-        """``term `` has only 1 child. Lift it up in place of the term."""
-        return quantified_or_atom
-
-    def visit_atom(self, atom, (child,)):
-        """Lift up the single child to replace this node."""
-        return child
-
     def visit_regex(self, regex, (tilde, literal, flags)):
         """Return a ``Regex`` expression."""
         flags = flags.text.upper()
@@ -342,13 +345,15 @@ class DslVisitor(NodeVisitor):
 
 
 # Bootstrap to level 1...
-dsl_grammar = BootstrappingGrammar(dsl_text)
-# ...and then to level 2. This establishes that the node tree of our grammar
-# DSL is built by the same machinery that will build that of our users'
+rule_grammar = BootstrappingGrammar(rule_syntax)
+# ...and then to level 2. This establishes that the node tree of our rule
+# syntax is built by the same machinery that will build trees of our users'
 # grammars. And the correctness of that tree is tested, indirectly, in
 # test_grammar.
-dsl_grammar = Grammar(dsl_text)
+rule_grammar = Grammar(rule_syntax)
 
 # TODO: Teach Expression trees how to spit out Python representations of
 # themselves. Then we can just paste that in about, and we won't have to
-# bootstrap on import. Though it'll be a little less DRY.
+# bootstrap on import. Though it'll be a little less DRY. [Ah, but this is not
+# so clean, because it would have to output multiple statements to get multiple
+# refs to a single expression hooked up.]
